"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const client_1 = require("@pythnetwork/client");
const web3_js_1 = require("@solana/web3.js");
const oracle_1 = require("../../src/accounts/oracle");
const utils_1 = require("../../src/utils");
const { MB_CLUSTER_URL } = process.env;
async function decodePrice(conn, ai, pk) {
    let uiPrice, price, lastUpdatedSlot, type;
    if ((0, oracle_1.isPythOracle)(ai)) {
        const priceData = (0, client_1.parsePriceData)(ai.data);
        uiPrice = priceData.previousPrice;
        price = (0, utils_1.toNativeI80F48)(uiPrice, 6 - 5);
        lastUpdatedSlot = parseInt(priceData.lastSlot.toString());
        type = 'pyth';
    }
    else if ((0, oracle_1.isSwitchboardOracle)(ai)) {
        const priceData = await (0, oracle_1.parseSwitchboardOracle)(pk, ai, conn);
        uiPrice = priceData.price;
        price = (0, utils_1.toNativeI80F48)(uiPrice, 6 - 5);
        lastUpdatedSlot = priceData.lastUpdatedSlot;
        type = 'sb';
    }
    console.log(`type ${type}`);
    console.log(`uiPrice ${uiPrice}`);
    console.log(`price ${price}`);
    console.log(`lastUpdatedSlot ${lastUpdatedSlot}`);
}
async function main() {
    try {
        const oraclePk1 = new web3_js_1.PublicKey('4SZ1qb4MtSUrZcoeaeQ3BDzVCyqxw3VwSFpPiMTmn4GE');
        const conn = new web3_js_1.Connection(MB_CLUSTER_URL);
        let ai = await conn.getAccountInfo(oraclePk1);
        decodePrice(conn, ai, oraclePk1);
        const oraclePk2 = new web3_js_1.PublicKey('8ihFLu5FimgTQ1Unh4dVyEHUGodJ5gJQCrQf4KUVB9bN');
        ai = await conn.getAccountInfo(oraclePk2);
        decodePrice(conn, ai, oraclePk2);
    }
    catch (error) {
        console.log(error);
    }
}
try {
    main();
}
catch (error) {
    console.log(error);
}
