"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@coral-xyz/anchor");
const anchor_2 = require("@project-serum/anchor");
const spl_governance_1 = require("@solana/spl-governance");
const web3_js_1 = require("@solana/web3.js");
const fs_1 = __importDefault(require("fs"));
const builder_1 = require("../src/builder");
const client_1 = require("../src/client");
const clientIxParamBuilder_1 = require("../src/clientIxParamBuilder");
const constants_1 = require("../src/constants");
const utils_1 = require("../src/utils");
const { MB_CLUSTER_URL, MB_PAYER_KEYPAIR } = process.env;
const CLIENT_USER = MB_PAYER_KEYPAIR;
const GROUP_PK = '78b8f4cGCwmZ9ysPFMWLaLTkkaYnUjwMJYStWe5RTSSX';
const defaultOracleConfig = {
    confFilter: 0.1,
    maxStalenessSlots: 120,
};
const defaultInterestRate = {
    adjustmentFactor: 0.004,
    util0: 0.7,
    rate0: 0.1,
    util1: 0.85,
    rate1: 0.2,
    maxRate: 2.0,
};
async function buildClient() {
    const clientKeypair = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(CLIENT_USER, 'utf-8'))));
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection(MB_CLUSTER_URL, options);
    const clientWallet = new anchor_1.Wallet(clientKeypair);
    const clientProvider = new anchor_1.AnchorProvider(connection, clientWallet, options);
    return await client_1.MangoClient.connect(clientProvider, 'mainnet-beta', constants_1.MANGO_V4_ID['mainnet-beta'], {
        idsSource: 'get-program-accounts',
    });
}
// async function groupEdit(): Promise<void> {
//   const client = await buildClient();
//   const group = await client.getGroup(new PublicKey(GROUP_PK));
//   const ix = await client.program.methods
//     .groupEdit(
//       null, // admin
//       null, // fastListingAdmin
//       null, // securityAdmin
//       null, // testing
//       null, // version
//       null, // depositLimitQuote
//       null, // feesPayWithMngo
//       null, // feesMngoBonusRate
//       null, // feesSwapMangoAccount
//       6, // feesMngoTokenIndex
//       null, // feesExpiryInterval
//       5, // allowedFastListingsPerInterval
//     )
//     .accounts({
//       group: group.publicKey,
//       admin: group.admin,
//     })
//     .instruction();
//   console.log(serializeInstructionToBase64(ix));
// }
// async function tokenRegister(): Promise<void> {
//   const client = await buildClient();
//   const group = await client.getGroup(new PublicKey(GROUP_PK));
//   const ix = await client.program.methods
//     .tokenRegister(
//       8 as TokenIndex,
//       'wBTC (Portal)',
//       defaultOracleConfig,
//       defaultInterestRate,
//       bpsToDecimal(50),
//       bpsToDecimal(5),
//       0.9,
//       0.8,
//       1.1,
//       1.2,
//       percentageToDecimal(5),
//       60 * 60,
//       0.06,
//       0.0003,
//       percentageToDecimal(20),
//       new BN(24 * 60 * 60),
//       new BN(toNative(50000, 6).toNumber()),
//       5_000_000_000,
//       5_000_000_000,
//       0,
//       0.0005,
//       0.0005,
//       0.0005,
//     )
//     .accounts({
//       group: group.publicKey,
//       admin: group.admin,
//       mint: new PublicKey('3NZ9JMVBmGAqocybic2c7LQCJScmgsAZ6vQqTDzcqmJh'),
//       oracle: new PublicKey('GVXRSBjFk6e6J3NbVPXohDJetcTjaeeuykUpbQF8UoMU'),
//       payer: (client.program.provider as AnchorProvider).wallet.publicKey,
//       rent: SYSVAR_RENT_PUBKEY,
//     })
//     .instruction();
//   // const coder = new BorshCoder(IDL);
//   // console.log(coder.instruction.decode(ix.data));
//   console.log(await serializeInstructionToBase64(ix));
// }
// async function tokenEdit(): Promise<void> {
//   const client = await buildClient();
//   const group = await client.getGroup(new PublicKey(GROUP_PK));
//   const params = Builder(NullTokenEditParams)
//     .borrowWeightScaleStartQuote(new BN(toNative(100000, 6)).toNumber())
//     .depositWeightScaleStartQuote(new BN(toNative(100000, 6)).toNumber())
//     .build();
//   const ix = await client.program.methods
//     .tokenEdit(
//       params.oracle,
//       params.oracleConfig,
//       params.groupInsuranceFund,
//       params.interestRateParams,
//       params.loanFeeRate,
//       params.loanOriginationFeeRate,
//       params.maintAssetWeight,
//       params.initAssetWeight,
//       params.maintLiabWeight,
//       params.initLiabWeight,
//       params.liquidationFee,
//       params.stablePriceDelayIntervalSeconds,
//       params.stablePriceDelayGrowthLimit,
//       params.stablePriceGrowthLimit,
//       params.minVaultToDepositsRatio,
//       params.netBorrowLimitPerWindowQuote !== null
//         ? new BN(params.netBorrowLimitPerWindowQuote)
//         : null,
//       params.netBorrowLimitWindowSizeTs !== null
//         ? new BN(params.netBorrowLimitWindowSizeTs)
//         : null,
//       params.borrowWeightScaleStartQuote,
//       params.depositWeightScaleStartQuote,
//       params.resetStablePrice ?? false,
//       params.resetNetBorrowLimit ?? false,
//       params.reduceOnly,
//       params.name,
//       params.forceClose,
//       params.tokenConditionalSwapTakerFeeRate,
//       params.tokenConditionalSwapMakerFeeRate,
//       params.flashLoanSwapFeeRate,
//     )
//     .accounts({
//       group: group.publicKey,
//       oracle: new PublicKey('GVXRSBjFk6e6J3NbVPXohDJetcTjaeeuykUpbQF8UoMU'),
//       admin: group.admin,
//       mintInfo: new PublicKey('59rgC1pa45EziDPyFgJgE7gbv7Dd7VaGmd2D93i1dtFk'),
//     })
//     .remainingAccounts([
//       {
//         pubkey: new PublicKey('8gabXzwdPn5TvtuQvysh3CxVbjfNY3TZd5XEG5qnueUm'),
//         isWritable: true,
//         isSigner: false,
//       } as AccountMeta,
//     ])
//     .instruction();
//   console.log(serializeInstructionToBase64(ix));
// }
// async function serum3Register(): Promise<void> {
//   const client = await buildClient();
//   const group = await client.getGroup(new PublicKey(GROUP_PK));
//   const ix = await client.program.methods
//     .serum3RegisterMarket(3, 'ETH (Portal)/USDC',)
//     .accounts({
//       group: group.publicKey,
//       admin: group.admin,
//       serumProgram: OPENBOOK_PROGRAM_ID['mainnet-beta'],
//       serumMarketExternal: new PublicKey(
//         'BbJgE7HZMaDp5NTYvRh5jZSkQPVDTU8ubPFtpogUkEj4',
//       ),
//       baseBank: group.getFirstBankByMint(
//         new PublicKey('7vfCXTUXx5WJV5JADk17DUJ4ksgau7utNKj4b963voxs'),
//       ).publicKey,
//       quoteBank: group.getFirstBankByMint(
//         new PublicKey('EPjFWdd5AufqSSqeM2qN1xzybapC8G4wEGGkZwyTDt1v'),
//       ).publicKey,
//       payer: (client.program.provider as AnchorProvider).wallet.publicKey,
//     })
//     .instruction();
//   console.log(serializeInstructionToBase64(ix));
// }
async function perpCreate() {
    const client = await buildClient();
    const bids = new web3_js_1.Keypair();
    const asks = new web3_js_1.Keypair();
    const eventQueue = new web3_js_1.Keypair();
    const bookSideSize = client.program._coder.accounts.size(client.program.account.bookSide._idlAccount);
    const eventQueueSize = client.program._coder.accounts.size(client.program.account.eventQueue._idlAccount);
    const group = await client.getGroup(new web3_js_1.PublicKey(GROUP_PK));
    const ix = await client.program.methods
        .perpCreateMarket(2, 'SOL-PERP', defaultOracleConfig, 9, new anchor_2.BN(100), new anchor_2.BN(10000000), 0.9, 0.8, 1.1, 1.2, 0, 0, (0, utils_1.percentageToDecimal)(5), (0, utils_1.bpsToDecimal)(-1), (0, utils_1.bpsToDecimal)(4), (0, utils_1.percentageToDecimal)(-5), (0, utils_1.percentageToDecimal)(5), new anchor_2.BN(100), false, (0, utils_1.bpsToDecimal)(0.5), (0, utils_1.toNative)(0.0001, 6).toNumber(), (0, utils_1.toNative)(100, 6).toNumber(), (0, utils_1.percentageToDecimal)(1), 0, 1, new anchor_2.BN(60 * 60), (0, utils_1.percentageToDecimal)(10), 0)
        .accounts({
        group: group.publicKey,
        admin: group.admin,
        oracle: new web3_js_1.PublicKey('H6ARHf6YXhGYeQfUzQNGk6rDNnLBQKrenN712K4AQJEG'),
        bids: bids.publicKey,
        asks: asks.publicKey,
        eventQueue: eventQueue.publicKey,
        payer: client.program.provider.wallet.publicKey,
    })
        .preInstructions([
        // book sides
        web3_js_1.SystemProgram.createAccount({
            programId: client.program.programId,
            space: bookSideSize,
            lamports: await client.program.provider.connection.getMinimumBalanceForRentExemption(bookSideSize),
            fromPubkey: client.program.provider.wallet
                .publicKey,
            newAccountPubkey: bids.publicKey,
        }),
        web3_js_1.SystemProgram.createAccount({
            programId: client.program.programId,
            space: bookSideSize,
            lamports: await client.program.provider.connection.getMinimumBalanceForRentExemption(bookSideSize),
            fromPubkey: client.program.provider.wallet
                .publicKey,
            newAccountPubkey: asks.publicKey,
        }),
        // event queue
        web3_js_1.SystemProgram.createAccount({
            programId: client.program.programId,
            space: eventQueueSize,
            lamports: await client.program.provider.connection.getMinimumBalanceForRentExemption(eventQueueSize),
            fromPubkey: client.program.provider.wallet
                .publicKey,
            newAccountPubkey: eventQueue.publicKey,
        }),
    ])
        .signers([bids, asks, eventQueue])
        .instruction();
    console.log((0, spl_governance_1.serializeInstructionToBase64)(ix));
}
async function perpEdit() {
    const client = await buildClient();
    const group = await client.getGroup(new web3_js_1.PublicKey(GROUP_PK));
    const perpMarket = group.getPerpMarketByMarketIndex(0);
    const params = (0, builder_1.Builder)(clientIxParamBuilder_1.NullPerpEditParams)
        .positivePnlLiquidationFee((0, utils_1.bpsToDecimal)(250))
        .build();
    const ix = await client.program.methods
        .perpEditMarket(params.oracle, params.oracleConfig, params.baseDecimals, params.maintBaseAssetWeight, params.initBaseAssetWeight, params.maintBaseLiabWeight, params.initBaseLiabWeight, params.maintOverallAssetWeight, params.initOverallAssetWeight, params.baseLiquidationFee, params.makerFee, params.takerFee, params.minFunding, params.maxFunding, params.impactQuantity !== null ? new anchor_2.BN(params.impactQuantity) : null, params.groupInsuranceFund, params.feePenalty, params.settleFeeFlat, params.settleFeeAmountThreshold, params.settleFeeFractionLowHealth, params.stablePriceDelayIntervalSeconds, params.stablePriceDelayGrowthLimit, params.stablePriceGrowthLimit, params.settlePnlLimitFactor, params.settlePnlLimitWindowSize !== null
        ? new anchor_2.BN(params.settlePnlLimitWindowSize)
        : null, params.reduceOnly, params.resetStablePrice ?? false, params.positivePnlLiquidationFee, params.name, params.forceClose, params.platformLiquidationFee)
        .accounts({
        group: group.publicKey,
        oracle: params.oracle ?? perpMarket.oracle,
        admin: group.admin,
        perpMarket: perpMarket.publicKey,
    })
        .instruction();
    console.log((0, spl_governance_1.serializeInstructionToBase64)(ix));
}
async function ixDisable() {
    const client = await buildClient();
    const group = await client.getGroup(new web3_js_1.PublicKey(GROUP_PK));
    const ixGateParams = clientIxParamBuilder_1.TrueIxGateParams;
    ixGateParams.HealthRegion = false;
    const ix = await client.program.methods
        .ixGateSet((0, clientIxParamBuilder_1.buildIxGate)(ixGateParams))
        .accounts({
        group: group.publicKey,
        admin: group.securityAdmin,
    })
        .instruction();
    console.log(await (0, spl_governance_1.serializeInstructionToBase64)(ix));
}
async function createMangoAccount() {
    const client = await buildClient();
    const group = await client.getGroup(new web3_js_1.PublicKey(GROUP_PK));
    const ix = await client.program.methods
        .accountCreate(0, 8, 8, 4, 32, 'Mango DAO 0')
        .accounts({
        group: group.publicKey,
        owner: new web3_js_1.PublicKey('5tgfd6XgwiXB9otEnzFpXK11m7Q7yZUaAJzWK4oT5UGF'),
        payer: new web3_js_1.PublicKey('5tgfd6XgwiXB9otEnzFpXK11m7Q7yZUaAJzWK4oT5UGF'),
    })
        .instruction();
    console.log(await (0, spl_governance_1.serializeInstructionToBase64)(ix));
}
async function idlResize() {
    // anchor constant for all idl-specific instructions
    const idlIxBytes = [0x40, 0xf4, 0xbc, 0x78, 0xa7, 0xe9, 0x69, 0x0a];
    const idlIxNum = 6; // resize
    const newSize = new anchor_2.BN(19000);
    const ix = new web3_js_1.TransactionInstruction({
        keys: [
            {
                pubkey: new web3_js_1.PublicKey('3foqXduY5PabCn6LjNrLo3waNf3Hy6vQgqavoVUCsUN9'),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: new web3_js_1.PublicKey('FP4PxqHTVzeG2c6eZd7974F9WvKUSdBeduUK3rjYyvBw'),
                isSigner: true,
                isWritable: true,
            },
            {
                pubkey: new web3_js_1.PublicKey('11111111111111111111111111111111'),
                isSigner: false,
                isWritable: false,
            },
        ],
        programId: constants_1.MANGO_V4_ID['mainnet-beta'],
        data: Buffer.from(idlIxBytes.concat([idlIxNum], newSize.toArray('le', 8))),
    });
    console.log(await (0, spl_governance_1.serializeInstructionToBase64)(ix));
}
async function idlSetAuthority() {
    // anchor constant for all idl-specific instructions
    const idlIxBytes = [0x40, 0xf4, 0xbc, 0x78, 0xa7, 0xe9, 0x69, 0x0a];
    const idlIxNum = 4; // setAuthority
    const newAuthority = new web3_js_1.PublicKey('8SSLjXBEVk9nesbhi9UMCA32uijbVBUqWoKPPQPTekzt');
    const ix = new web3_js_1.TransactionInstruction({
        keys: [
            {
                pubkey: new web3_js_1.PublicKey('3foqXduY5PabCn6LjNrLo3waNf3Hy6vQgqavoVUCsUN9'),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: new web3_js_1.PublicKey('FP4PxqHTVzeG2c6eZd7974F9WvKUSdBeduUK3rjYyvBw'),
                isSigner: true,
                isWritable: true,
            },
        ],
        programId: constants_1.MANGO_V4_ID['mainnet-beta'],
        data: Buffer.concat([
            Buffer.from(idlIxBytes.concat([idlIxNum])),
            newAuthority.toBuffer(),
        ]),
    });
    console.log(await (0, spl_governance_1.serializeInstructionToBase64)(ix));
}
async function main() {
    try {
        // await groupEdit();
        // await tokenRegister();
        // await tokenEdit();
        // await perpCreate();
        // await perpEdit();
        // await serum3Register();
        // await ixDisable();
        // await createMangoAccount();
        // await idlResize();
        // await idlSetAuthority();
    }
    catch (error) {
        console.log(error);
    }
}
try {
    main();
}
catch (error) {
    console.log(error);
}
