"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@coral-xyz/anchor");
const anchor_2 = require("@project-serum/anchor");
const web3_js_1 = require("@solana/web3.js");
const fs_1 = __importDefault(require("fs"));
const client_1 = require("../src/client");
const constants_1 = require("../src/constants");
const { MB_CLUSTER_URL, MB_PAYER_KEYPAIR, MANGO_ACCOUNT, MINT, NATIVE_AMOUNT } = process.env;
const CLIENT_USER = MB_PAYER_KEYPAIR;
const GROUP_PK = '78b8f4cGCwmZ9ysPFMWLaLTkkaYnUjwMJYStWe5RTSSX';
async function buildClient() {
    const clientKeypair = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(CLIENT_USER, 'utf-8'))));
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection(MB_CLUSTER_URL, options);
    const clientWallet = new anchor_1.Wallet(clientKeypair);
    const clientProvider = new anchor_1.AnchorProvider(connection, clientWallet, options);
    return await client_1.MangoClient.connect(clientProvider, 'mainnet-beta', constants_1.MANGO_V4_ID['mainnet-beta'], {
        idsSource: 'get-program-accounts',
    });
}
async function main() {
    const client = await buildClient();
    const mangoAccount = await client.getMangoAccount(new web3_js_1.PublicKey(MANGO_ACCOUNT));
    const group = await client.getGroup(new web3_js_1.PublicKey(GROUP_PK));
    const mintPk = new web3_js_1.PublicKey(MINT);
    const rs = await client.tokenDepositNative(group, mangoAccount, mintPk, new anchor_2.BN(NATIVE_AMOUNT), false, true);
    console.log(rs.signature);
}
try {
    main();
}
catch (error) {
    console.log(error);
}
