"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@coral-xyz/anchor");
const web3_js_1 = require("@solana/web3.js");
const fs = __importStar(require("fs"));
const src_1 = require("../src");
const outputCsvName = 'mango-switchboard-snapshot.csv';
const script = async () => {
    const connection = new web3_js_1.Connection(process.env.MB_CLUSTER_URL, 'confirmed');
    const kp = web3_js_1.Keypair.generate();
    const wallet = new anchor_1.Wallet(kp);
    const provider = new anchor_1.AnchorProvider(connection, wallet, {});
    const CLUSTER = 'mainnet-beta';
    const client = src_1.MangoClient.connect(provider, CLUSTER, src_1.MANGO_V4_ID[CLUSTER], { idsSource: 'api' });
    const groupKey = new web3_js_1.PublicKey('78b8f4cGCwmZ9ysPFMWLaLTkkaYnUjwMJYStWe5RTSSX');
    const group = await client.getGroup(groupKey);
    const mangoAccounts = await client.getAllMangoAccounts(group, false);
    const sbTokenIndexes = [
        // https://api.mngo.cloud/data/v4/group-metadata
        791,
        916,
        881,
        889,
        743,
        669,
        455,
        616,
        848, // WEN
    ];
    const sbBanks = await Promise.all(sbTokenIndexes.map((i) => group.getFirstBankByTokenIndex(i)));
    function accountHasSbTokens(m) {
        for (const bank of sbBanks) {
            if (m.getTokenBalanceUi(bank) !== 0) {
                return true;
            }
        }
        return false;
    }
    const sbAccounts = mangoAccounts.filter((m) => accountHasSbTokens(m));
    const resultMap = new Map();
    for (const account of sbAccounts) {
        let deposits = 0;
        let borrows = 0;
        for (const bank of sbBanks) {
            deposits += account.getTokenDepositsUi(bank) * bank.uiPrice;
            borrows += account.getTokenBorrowsUi(bank) * bank.uiPrice;
        }
        const wallet = account.owner.toBase58();
        const existingValue = resultMap.get(account.owner.toBase58());
        if (existingValue) {
            const [d, b] = existingValue;
            resultMap.set(wallet, [d + deposits, b + borrows]);
        }
        else {
            resultMap.set(wallet, [deposits, borrows]);
        }
    }
    // console.log(resultMap.entries())
    writeMapToCsv(resultMap, outputCsvName);
};
script();
function writeMapToCsv(map, filename) {
    let csv = 'wallet,deposits_value,borrows_value\n';
    for (let [wallet, [deposits, borrows]] of map) {
        csv += `${wallet},${deposits},${borrows}\n`;
    }
    fs.writeFileSync(filename, csv);
}
