"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const oracle_1 = require("../src/accounts/oracle");
const client_1 = require("../src/client");
const constants_1 = require("../src/constants");
const { MB_CLUSTER_URL } = process.env;
async function buildClient() {
    return await client_1.MangoClient.connectDefault(MB_CLUSTER_URL);
}
async function updateSpotMarkets() {
    const [client] = await Promise.all([buildClient()]);
    const group = await client.getGroup(constants_1.MANGO_V4_MAIN_GROUP);
    Array.from(group.banksMapByTokenIndex.values())
        .map((banks) => banks[0])
        .sort((a, b) => a.name.localeCompare(b.name))
        .forEach(async (bank) => {
        // https://pyth.network/developers/price-feed-ids, use pyth evm stable
        // https://docs.pyth.network/price-feeds/sponsored-feeds
        const sponsored = [
            'SOL',
            'JITOSOL',
            'MSOL',
            'BSOL',
            'BONK',
            'W',
            'KMNO',
            'MEW',
            'TNSR',
            'USDC',
            'BTC',
            'JTO',
            'USDT',
            'JUP',
            'ETH',
            'PYTH',
            'HNT',
            'RNDR',
            'ORCA',
            'SAMO',
            'WIF',
            'LST',
            'INF',
            'PRCL',
            'RAY',
            'FIDA',
            'MNDE',
            'MOBILE',
            'IOT',
            'GOFX',
            'NEON',
            'AUD',
            'GBP',
            'EUR',
            'XAG',
            'XAU',
        ];
        if (bank.oracleProvider == oracle_1.OracleProvider.Pyth) {
            let bankName = bank.name;
            if (bankName == 'ETH (Portal)') {
                bankName = 'ETH';
            }
            if (bankName == 'wBTC (Portal)') {
                bankName = 'BTC';
            }
            console.log(`${bank.name}, ${bank.oracle}, is sponsored ${sponsored.filter((t) => t.toLowerCase() == bankName.toLowerCase())
                .length > 0}`);
        }
    });
}
async function main() {
    try {
        await updateSpotMarkets();
    }
    catch (error) {
        console.log(error);
    }
}
try {
    main();
}
catch (error) {
    console.log(error);
}
