"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFallbackOracleMap = exports.deriveFallbackOracleQuoteKey = exports.isOracleStaleOrUnconfident = exports.isClmmOracle = exports.isRaydiumOracle = exports.isOrcaOracle = exports.isPythOracle = exports.isSwitchboardOracle = exports.parseSwitchboardOracle = exports.parseSwitchboardOracleV2 = exports.switchboardDecimalToBig = exports.parseSwitchboardOracleV1 = exports.StubOracle = exports.OracleProvider = exports.SOL_MINT_MAINNET = exports.USDC_MINT_MAINNET = void 0;
const client_1 = require("@pythnetwork/client");
const web3_js_1 = require("@solana/web3.js");
const sbv2_lite_1 = __importDefault(require("@switchboard-xyz/sbv2-lite"));
const big_js_1 = __importDefault(require("big.js"));
const I80F48_1 = require("../numbers/I80F48");
const SBV1_DEVNET_PID = new web3_js_1.PublicKey('7azgmy1pFXHikv36q1zZASvFq5vFa39TT9NweVugKKTU');
const SBV1_MAINNET_PID = new web3_js_1.PublicKey('DtmE9D2CSB4L5D6A15mraeEjrGMm6auWVzgaD8hK2tZM');
const ORCA_MAINNET_PID = new web3_js_1.PublicKey('whirLbMiicVdio4qvUfM5KAg6Ct8VwpYzGff3uctyCc');
const ORCA_WHIRLPOOL_LEN = 653;
const ORCA_WHIRLPOOL_DISCRIMINATOR = [63, 149, 209, 12, 225, 128, 99, 9];
const RAYDIUM_MAINNET_PID = new web3_js_1.PublicKey('CAMMCzo5YL8w4VFF8KVHrK22GGUsp5VTaW7grrKgrWqK');
const RAYDIUM_POOL_LEN = 1544;
const RAYDIUM_POOL_DISCRIMINATOR = [247, 237, 227, 245, 215, 195, 222, 70];
exports.USDC_MINT_MAINNET = new web3_js_1.PublicKey('EPjFWdd5AufqSSqeM2qN1xzybapC8G4wEGGkZwyTDt1v');
exports.SOL_MINT_MAINNET = new web3_js_1.PublicKey('So11111111111111111111111111111111111111112');
let sbv2DevnetProgram;
let sbv2MainnetProgram;
var OracleProvider;
(function (OracleProvider) {
    OracleProvider[OracleProvider["Pyth"] = 0] = "Pyth";
    OracleProvider[OracleProvider["Switchboard"] = 1] = "Switchboard";
    OracleProvider[OracleProvider["Stub"] = 2] = "Stub";
})(OracleProvider = exports.OracleProvider || (exports.OracleProvider = {}));
class StubOracle {
    publicKey;
    group;
    mint;
    lastUpdateTs;
    lastUpdateSlot;
    price;
    deviation;
    static from(publicKey, obj) {
        return new StubOracle(publicKey, obj.group, obj.mint, obj.price, obj.lastUpdateTs, obj.lastUpdateSlot, obj.deviation);
    }
    constructor(publicKey, group, mint, price, lastUpdateTs, lastUpdateSlot, deviation) {
        this.publicKey = publicKey;
        this.group = group;
        this.mint = mint;
        this.lastUpdateTs = lastUpdateTs;
        this.lastUpdateSlot = lastUpdateSlot;
        this.price = I80F48_1.I80F48.from(price);
        this.deviation = I80F48_1.I80F48.from(deviation);
    }
}
exports.StubOracle = StubOracle;
// https://gist.github.com/microwavedcola1/b741a11e6ee273a859f3ef00b35ac1f0
function parseSwitchboardOracleV1(accountInfo) {
    const price = accountInfo.data.readDoubleLE(1 + 32 + 4 + 4);
    const lastUpdatedSlot = parseInt(accountInfo.data.readBigUInt64LE(1 + 32 + 4 + 4 + 8).toString());
    const minResponse = accountInfo.data.readDoubleLE(1 + 32 + 4 + 4 + 8 + 8 + 8);
    const maxResponse = accountInfo.data.readDoubleLE(1 + 32 + 4 + 4 + 8 + 8 + 8 + 8);
    return { price, lastUpdatedSlot, uiDeviation: maxResponse - minResponse };
}
exports.parseSwitchboardOracleV1 = parseSwitchboardOracleV1;
function switchboardDecimalToBig(sbDecimal) {
    const mantissa = new big_js_1.default(sbDecimal.mantissa.toString());
    const scale = sbDecimal.scale;
    const oldDp = big_js_1.default.DP;
    big_js_1.default.DP = 20;
    const result = mantissa.div(new big_js_1.default(10).pow(scale));
    big_js_1.default.DP = oldDp;
    return result;
}
exports.switchboardDecimalToBig = switchboardDecimalToBig;
function parseSwitchboardOracleV2(program, accountInfo, oracle) {
    try {
        //
        const price = program.decodeLatestAggregatorValue(accountInfo).toNumber();
        const lastUpdatedSlot = program
            .decodeAggregator(accountInfo)
            .latestConfirmedRound.roundOpenSlot.toNumber();
        const stdDeviation = switchboardDecimalToBig(program.decodeAggregator(accountInfo).latestConfirmedRound.stdDeviation);
        return { price, lastUpdatedSlot, uiDeviation: stdDeviation.toNumber() };
        //if oracle is badly configured or didn't publish price at least once
        //decodeLatestAggregatorValue can throw (0 switchboard rounds).
    }
    catch (e) {
        console.log(`Unable to parse Switchboard Oracle V2: ${oracle}`, e);
        return { price: 0, lastUpdatedSlot: 0, uiDeviation: 0 };
    }
}
exports.parseSwitchboardOracleV2 = parseSwitchboardOracleV2;
/**
 *
 * @param accountInfo
 * @returns ui price
 */
async function parseSwitchboardOracle(oracle, accountInfo, connection) {
    if (accountInfo.owner.equals(sbv2_lite_1.default.devnetPid)) {
        if (!sbv2DevnetProgram) {
            sbv2DevnetProgram = await sbv2_lite_1.default.loadDevnet(connection);
        }
        return parseSwitchboardOracleV2(sbv2DevnetProgram, accountInfo, oracle);
    }
    if (accountInfo.owner.equals(sbv2_lite_1.default.mainnetPid)) {
        if (!sbv2MainnetProgram) {
            sbv2MainnetProgram = await sbv2_lite_1.default.loadMainnet(connection);
        }
        return parseSwitchboardOracleV2(sbv2MainnetProgram, accountInfo, oracle);
    }
    if (accountInfo.owner.equals(SBV1_DEVNET_PID) ||
        accountInfo.owner.equals(SBV1_MAINNET_PID)) {
        return parseSwitchboardOracleV1(accountInfo);
    }
    throw new Error(`Should not be reached!`);
}
exports.parseSwitchboardOracle = parseSwitchboardOracle;
function isSwitchboardOracle(accountInfo) {
    if (accountInfo.owner.equals(SBV1_DEVNET_PID) ||
        accountInfo.owner.equals(SBV1_MAINNET_PID) ||
        accountInfo.owner.equals(sbv2_lite_1.default.devnetPid) ||
        accountInfo.owner.equals(sbv2_lite_1.default.mainnetPid)) {
        return true;
    }
    return false;
}
exports.isSwitchboardOracle = isSwitchboardOracle;
function isPythOracle(accountInfo) {
    return accountInfo.data.readUInt32LE(0) === client_1.Magic;
}
exports.isPythOracle = isPythOracle;
function isOrcaOracle(accountInfo) {
    for (let i = 0; i < 8; i++) {
        if (accountInfo.data.at(i) !== ORCA_WHIRLPOOL_DISCRIMINATOR[i]) {
            return false;
        }
    }
    return (accountInfo.owner.equals(ORCA_MAINNET_PID) &&
        accountInfo.data.length == ORCA_WHIRLPOOL_LEN);
}
exports.isOrcaOracle = isOrcaOracle;
function isRaydiumOracle(accountInfo) {
    for (let i = 0; i < 8; i++) {
        if (accountInfo.data.at(i) !== RAYDIUM_POOL_DISCRIMINATOR[i]) {
            return false;
        }
    }
    return (accountInfo.owner.equals(RAYDIUM_MAINNET_PID) &&
        accountInfo.data.length == RAYDIUM_POOL_LEN);
}
exports.isRaydiumOracle = isRaydiumOracle;
function isClmmOracle(accountInfo) {
    return isOrcaOracle(accountInfo) || isRaydiumOracle(accountInfo);
}
exports.isClmmOracle = isClmmOracle;
function isOracleStaleOrUnconfident(nowSlot, maxStalenessSlots, oracleLastUpdatedSlot, deviation, confFilter, price) {
    if (maxStalenessSlots >= 0 &&
        oracleLastUpdatedSlot &&
        nowSlot > oracleLastUpdatedSlot + maxStalenessSlots) {
        return true;
    }
    if (deviation && deviation.gt(confFilter.mul(price))) {
        return true;
    }
    return false;
}
exports.isOracleStaleOrUnconfident = isOracleStaleOrUnconfident;
function deriveFallbackOracleQuoteKey(accountInfo) {
    if (isOrcaOracle(accountInfo)) {
        const tokenA = new web3_js_1.PublicKey(accountInfo.data.subarray(101, 133));
        const tokenB = new web3_js_1.PublicKey(accountInfo.data.subarray(181, 213));
        return clmmQuoteKey(tokenA, tokenB);
    }
    else if (isRaydiumOracle(accountInfo)) {
        const tokenA = new web3_js_1.PublicKey(accountInfo.data.subarray(73, 105));
        const tokenB = new web3_js_1.PublicKey(accountInfo.data.subarray(105, 137));
        return clmmQuoteKey(tokenA, tokenB);
    }
    else {
        return web3_js_1.PublicKey.default;
    }
}
exports.deriveFallbackOracleQuoteKey = deriveFallbackOracleQuoteKey;
function clmmQuoteKey(tokenA, tokenB) {
    if (tokenA.equals(exports.USDC_MINT_MAINNET) ||
        (tokenA.equals(exports.SOL_MINT_MAINNET) && !tokenB.equals(exports.USDC_MINT_MAINNET))) {
        return tokenA; // inverted
    }
    else {
        return tokenB;
    }
}
// Assumes oracles.length === fallbacks.length
async function createFallbackOracleMap(conn, oracles, fallbacks) {
    const map = new Map();
    const accounts = await conn.getMultipleAccountsInfo(fallbacks);
    for (let i = 0; i < oracles.length; i++) {
        if (accounts[i] === null) {
            map.set(oracles[i].toBase58(), [fallbacks[i], web3_js_1.PublicKey.default]);
        }
        else if (!isClmmOracle(accounts[i])) {
            map.set(oracles[i].toBase58(), [fallbacks[i], web3_js_1.PublicKey.default]);
        }
        else {
            const quoteKey = deriveFallbackOracleQuoteKey(accounts[i]);
            map.set(oracles[i].toBase58(), [fallbacks[i], quoteKey]);
        }
    }
    return map;
}
exports.createFallbackOracleMap = createFallbackOracleMap;
